import ViewEngine from './view-engine';

/**
 * 编辑视图引擎
 *
 * @export
 * @class EditViewEngine
 * @extends {ViewEngine}
 */
export default class EditViewEngine extends ViewEngine {

    /**
     * 表单部件
     *
     * @protected
     * @type {*}
     * @memberof EditViewEngine
     */
    protected form: any;

    /**
     * 父健为当前健
     *
     * @protected
     * @type {string}
     * @memberof EditViewEngine
     */
    protected p2k: string = '';

    /**
     * 初始化编辑视图引擎
     *
     * @param {*} [options={}]
     * @memberof EditViewEngine
     */
    public init(options: any = {}): void {
        this.form = options.form;
        this.p2k = options.p2k;
        super.init(options);
    }

    /**
     * 引擎加载
     *
     * @param {*} [opts={}]
     * @memberof EditViewEngine
     */
    public load(opts: any = {}): void {
        super.load(opts);
        if (this.getForm()) {
            const tag = this.getForm().name;

            const data: any = {};
            let srfkey: string = this.viewdata.srfkey;
            let srfkeys: string = this.viewdata.srfkeys;
            let srfsourcekey: string = this.viewdata.srfsourcekey;
            let action: string = '';

            const hasSrfParentKey = this.viewdata.srfparentdata
                && this.viewdata.srfparentdata.srfparentkey
                && !Object.is(this.viewdata.srfparentdata.srfparentkey, '');
            if (Object.is(this.p2k, '1') && hasSrfParentKey) {
                srfkey = this.viewdata.srfparentdata.srfparentkey;
            }
            if (srfkey && !Object.is(srfkey, '')) {
                Object.assign(data, this.viewdata.srfparentdata);
                action = 'load';
            } else {
                Object.assign(data, { srfparentdata: this.viewdata.srfparentdata });
                Object.assign(data, { srfsourcekey: srfsourcekey, srfkeys: srfkeys });
                action = 'loaddraft';
            }

            Object.assign(data, { srfkey: srfkey });
            this.setViewState2({ tag: tag, action: action, viewdata: data });
        }
    }

    /**
     * 部件事件机制
     *
     * @param {string} ctrlName
     * @param {string} eventName
     * @param {*} args
     * @memberof EditViewEngine
     */
    public onCtrlEvent(ctrlName: string, eventName: string, args: any): void {
        super.onCtrlEvent(ctrlName, eventName, args);
        if (Object.is(ctrlName, 'form')) {
            this.formEvent(eventName, args);
        }
    }

    /**
     * 表单事件
     *
     * @param {string} eventName
     * @param {*} args
     * @memberof EditViewEngine
     */
    public formEvent(eventName: string, args: any): void {
        if (Object.is(eventName, 'load')) {
            this.onFormLoad(args);
        }
        if (Object.is(eventName, 'save')) {
            this.onFormSave(args);
        }
        if (Object.is(eventName, 'remove')) {
            this.onFormRemove(args);
        }
    }

    /**
     * 表单加载完成
     *
     * @param {*} args
     * @memberof EditViewEngine
     */
    public onFormLoad(arg: any): void {
       this.view.$emit('load',arg);
       this.view.$emit('viewdataschange',{action:'load',data:arg});
    }

    /**
     * 表单保存完成
     *
     * @param {*} args
     * @memberof EditViewEngine
     */
    public onFormSave(arg: any): void {
        this.view.$emit('save',arg);
        this.view.$emit('viewdataschange',{action:'save',data:arg});
    }

    /**
     * 表单删除完成
     *
     * @param {*} args
     * @memberof EditViewEngine
     */
    public onFormRemove(arg: any): void {
        this.view.$emit('remove',arg);
        this.view.$emit('viewdataschange',{action:'remove',data:arg});
    }

    /**
     * 处理实体界面行为
     *
     * @param {string} tag
     * @param {string} [actionmode]
     * @returns {void}
     * @memberof EditViewEngine
     */
    public doSysUIAction(tag: string, actionmode?: string): void {
        if (Object.is(tag, 'Save')) {
            this.doSave();
            return;
        }
        if (Object.is(tag, 'SaveAndExit')) {
            this.doSaveAndExit();
            return;
        }
        if (Object.is(tag, 'SaveAndNew')) {
            this.doSaveAndNew();
            return;
        }
        if (Object.is(tag, 'RemoveAndExit')) {
            this.doRemoveAndExit(this.viewdata);
            return;
        }
        if (Object.is(tag, 'Refresh')) {
            this.doRefresh();
            return;
        }
        if (Object.is(tag, 'SaveAndStart')) {
            this.doSaveAndStart();
            return;
        }
        super.doSysUIAction(tag, actionmode);
    }

    /**
     * 编辑界面_保存操作
     * 
     * @memberof EditViewEngine
     */
    public doSave(): void {
        this.saveData({});
    }

     /**
     * 编辑界面_保存并开始流程操作
     * 
     * @memberof EditViewEngine
     */
    public doSaveAndStart(){
        this.SaveAndStart({action:'saveandstart'});
    }

    /**
     * 编辑界面_保存并退出操作
     * 
     * @memberof EditViewEngine
     */
    public doSaveAndExit():void{
        this.SaveAndExit({});
    }

    /**
     * 编辑界面_保存并新建操作
     * 
     * @memberof EditViewEngine
     */
    public doSaveAndNew():void{
        this.SaveAndNew({});
    }

    /**
     * 编辑界面_删除并关闭操作
     * 
     * @memberof EditViewEngine
     */
    public doRemoveAndExit(data:any):void{
        this.RemoveAndExit(data);
    }

    /**
     * 编辑界面_刷新操作
     * 
     * @memberof EditViewEngine
     */
    public doRefresh():void{
        this.Refresh({});
    }

    /**
     * 编辑界面_保存并开始流程操作
     * 
     * @memberof EditViewEngine
     */
    public SaveAndStart(arg: any = {}):void{
        if (this.getForm()) {
            const tag = this.getForm().name;
            this.setViewState2({ tag: tag, action: 'saveandstart', viewdata: arg });
        }
    }

    /**
     * 保存视图数据
     *
     * @param {*} [arg={}]
     * @memberof EditViewEngine
     */
    public saveData(arg: any = {}): void {
        if (this.getForm()) {
            const tag = this.getForm().name;
            this.setViewState2({ tag: tag, action: 'save', viewdata: arg });
        }
    }

    /**
     * 保存并退出操作
     * 
     * @memberof EditViewEngine
     */
    public SaveAndExit(arg: any = {}):void{
        if (this.getForm()) {
            const tag = this.getForm().name;
            this.setViewState2({ tag: tag, action: 'saveandexit', viewdata: arg });
        }
    }

    /**
     * 删除并关闭
     * 
     * @memberof EditViewEngine
     */
    public RemoveAndExit(arg: any = {}):void{
        if (this.getForm()) {
            const tag = this.getForm().name;
            this.setViewState2({ tag: tag, action: 'removeandexit', viewdata: arg });
        }
    }

    /**
     * 保存并新建操作
     * 
     * @memberof EditViewEngine
     */
    public SaveAndNew(arg: any = {}):void{
        if (this.getForm()) {
            const tag = this.getForm().name;
            this.setViewState2({ tag: tag, action: 'saveandnew', viewdata: arg });
        }
    }

    /**
     * 刷新
     * 
     * @memberof EditViewEngine
     */
    public Refresh(arg: any = {}):void{
        if (this.getForm()) {
            const tag = this.getForm().name;
            this.setViewState2({ tag: tag, action: 'refresh', viewdata: arg });
        }
    }

    /**
     * 获取表单对象
     *
     * @returns {*}
     * @memberof EditViewEngine
     */
    public getForm(): any {
        return this.form;
    }

}