import ViewEngine from './view-engine';

/**
 * 实体选择视图
 *
 * @export
 * @class MobPickupViewEngine
 * @extends {ViewEngine}
 */
export default class MobPickupViewEngine extends ViewEngine {

    /**
     * 选择视图面板
     *
     * @type {*}
     * @memberof MobPickupViewEngine
     */
    public pickupViewPanel: any = null;

    /**
     * Creates an instance of MobPickupViewEngine.
     * 
     * @memberof MobPickupViewEngine
     */
    constructor() {
        super();
    }

    /**
     * 初始化引擎
     *
     * @param {*} options
     * @memberof MobPickupViewEngine
     */
    public init(options: any): void {
        this.pickupViewPanel = options.pickupviewpanel;
        super.init(options);
    }


    /**
     * 引擎加载
     *
     * @memberof MobPickupViewEngine
     */
    public load(): void {
        this.view.viewSelections = [];
        super.load();
        const _srfparentdata = this.viewdata.srfparentdata ? { srfparentdata: this.viewdata.srfparentdata } : { srfparentdata: {} };
        if (this.getPickupViewPanel()) {
            const tag = this.getPickupViewPanel().name;
            this.setViewState2({ tag: tag, action: 'load', viewdata: _srfparentdata });
        }
    }

    /**
     * 
     *
     * @param {string} ctrlName
     * @param {string} eventName
     * @param {*} args
     * @memberof MobPickupViewEngine
     */
    public onCtrlEvent(ctrlName: string, eventName: string, args: any): void {
        super.onCtrlEvent(ctrlName, eventName, args);

        if (Object.is(eventName, 'dataschange')) {
            this.onSelectionChange(ctrlName, args);
        }
        if (Object.is(eventName, 'activated')) {
            this.view.$emit('viewdatasactivated', args);
        }
    }

    /**
     * 值选中变化
     *
     * @param {any[]} args
     * @memberof MobPickupViewEngine
     */
    public onSelectionChange(ctrlName:any,args: any[]): void {
        this.view.containerModel[`view_${ctrlName}`].selections = args;
         this.view.viewSelections = [];
         this.view.viewSelections.push(args);
         const _disabled: boolean = this.view.viewSelections.length > 0 ? false : true;
         this.view.containerModel.view_okbtn.disabled = _disabled;
    }

    /**
     * 获取选择视图面板
     *
     * @returns {*}
     * @memberof MobPickupViewEngine
     */
    public getPickupViewPanel(): any {
        return this.pickupViewPanel;
    }
}